/*
 *  libpipi       Proper image processing implementation library
 *  Copyright (c) 2004-2008 Sam Hocevar <sam@zoy.org>
 *                All Rights Reserved
 *
 *  $Id$
 *
 *  This library is free software. It comes without any warranty, to
 *  the extent permitted by applicable law. You can redistribute it
 *  and/or modify it under the terms of the Do What The Fuck You Want
 *  To Public License, Version 2, as published by Sam Hocevar. See
 *  http://sam.zoy.org/wtfpl/COPYING for more details.
 */

/*
 * measure.c: distance functions
 */

#include "config.h"
#include "common.h"

#include <math.h>

#include "pipi.h"
#include "pipi_internals.h"

double pipi_measure_rmsd(pipi_image_t *i1, pipi_image_t *i2)
{
    return sqrt(pipi_measure_msd(i1, i2));
}

double pipi_measure_msd(pipi_image_t *i1, pipi_image_t *i2)
{
    pipi_format_t f1, f2;
    double ret = 0.0;
    float *p1, *p2;
    int x, y, w, h;

    w = i1->w < i2->w ? i1->w : i2->w;
    h = i1->h < i2->h ? i1->h : i2->h;

    f1 = i1->last_modified;
    f2 = i2->last_modified;

    pipi_getpixels(i1, PIPI_PIXELS_Y_F);
    pipi_getpixels(i2, PIPI_PIXELS_Y_F);

    p1 = (float *)i1->p[PIPI_PIXELS_Y_F].pixels;
    p2 = (float *)i2->p[PIPI_PIXELS_Y_F].pixels;

    for(y = 0; y < h; y++)
        for(x = 0; x < w; x++)
        {
            float a = p1[y * i1->w + x];
            float b = p2[y * i2->w + x];
            ret += (a - b) * (a - b);
        }

    /* TODO: free pixels if they were allocated */

    /* Restore original image formats */
    i1->last_modified = f1;
    i2->last_modified = f2;

    return ret / (w * h);
}

